@@ -37,6 +37,9 @@
 #'   sets of formant values than the number of fft steps.
 #'   \code{smoothLinearFactor} = 0: close to default spline; >3: approaches
 #'   linear extrapolation
+#' @param output "simple" returns just the spectral filter, while "detailed"
+#'   also returns a data.frame of formant frequencies over time (needed for
+#'   internal purposes such as formant locking)
 #' @param plot if TRUE, produces a plot of the spectral envelope
 #' @param duration duration of the sound, ms (for plotting purposes only)
 #' @param colorTheme black and white ('bw'), as in seewave package ('seewave'),
@@ -119,6 +122,7 @@ getSpectralEnvelope = function(nr,
                                formantCeiling = 2,
                                samplingRate = 16000,
                                speedSound = 35400,
+                               output = c('simple', 'detailed')[1],
                                plot = FALSE,
                                duration = NULL,
                                colorTheme = c('bw', 'seewave', '...')[1],
@@ -572,7 +576,24 @@ getSpectralEnvelope = function(nr,
           ...)
   }
 
-  invisible(spectralEnvelope_lin)
+  if (output == 'detailed') {
+    if (exists('formants_upsampled')) {
+      formantSummary = as.data.frame(matrix(NA, nrow = nFormants, ncol = nc))
+      for (i in 1:nFormants) {
+        formantSummary[i, ] = (formants_upsampled[[i]]$freq - 1) *
+          bin_width + bin_width / 2  # from bins back to Hz
+      }
+      max_freqs = apply(formantSummary, 1, max)  # save only to Nyquist
+      formantSummary = formantSummary[which(max_freqs < (samplingRate / 2)), ]
+      colnames(formantSummary) = colnames(spectralEnvelope)
+    } else {
+      formantSummary = NULL
+    }
+    invisible(list(formantSummary = formantSummary,
+                   specEnv = spectralEnvelope_lin))
+  } else {
+    invisible(spectralEnvelope_lin)
+  }
 }
 
 
@@ -906,20 +927,25 @@ addFormants = function(sound,
     nr = windowLength_points / 2 # number of frequency bins for fft
 
     # are formants moving or stationary?
+    # (basically always moving, unless temperature = 0 and nothing else changes)
     if (is.null(spectralEnvelope)) {
-      if (is.list(formants)) {
-        movingFormants = max(sapply(formants, function(x) sapply(x, length))) > 1
+      if (temperature > 0) {
+        movingFormants = TRUE
       } else {
-        movingFormants = FALSE
-      }
-      if (is.list(mouth)) {
-        if (sum(mouth$value != .5) > 0) {
-          movingFormants = TRUE
+        if (is.list(formants)) {
+          movingFormants = max(sapply(formants, function(x) sapply(x, length))) > 1
+        } else {
+          movingFormants = FALSE
         }
-      }
-      if (is.list(vocalTract)) {
-        if (length(vocalTract$value) > 1) {
-          movingFormants = TRUE
+        if (is.list(mouth)) {
+          if (sum(mouth$value != .5) > 0) {
+            movingFormants = TRUE
+          }
+        }
+        if (is.list(vocalTract)) {
+          if (length(vocalTract$value) > 1) {
+            movingFormants = TRUE
+          }
         }
       }
       nInt = ifelse(movingFormants, nc, 1)
@@ -1278,3 +1304,159 @@ transplantFormants = function(donor,
   # spectrogram(recipient_new, samplingRate)
   return(recipient_new)
 }
+
+#' Lock to formants
+#'
+#' Internal soundgen function
+#'
+#' When f0 or another relatively strong harmonic is close to one of the
+#' formants, the pitch contour is modified so as to "lock" it to this formant.
+#' The relevant metric is energy gain (ratio of amplitudes before and after the
+#' adjustment) penalized by the magnitude of the necessary pitch jump (in
+#' semitones) and the amplitude of the locked harmonic relative to f0.
+#' @param pitch pitch contour, numeric vector (normally pitch_per_gc)
+#' @param specEnv spectral envelope as returned by getSpectralEnvelope
+#' @param rolloffMatrix rolloff matrix as returned by getRolloff
+#' @param formantSummary matrix of exact formant frequencies (formants in rows,
+#'   time in columns)
+#' @param lockProb the (approximate) proportion of sound affected by formant
+#'   locking
+#' @param minLength the minimum number of consecutive pitch values affected
+#'   (shorter segments of formant locking are ignored)
+#' @param plot if TRUE, plots the original and modified pitch contour
+#' @keywords internal
+#' @examples
+#' n = 50
+#' pitch = getSmoothContour(len = n, anchors = c(600, 2000, 1900, 400),
+#'   thisIsPitch = TRUE, plot = TRUE)
+#' rolloffMatrix = getRolloff(pitch_per_gc = pitch)
+#' specEnv = getSpectralEnvelope(nr = 512, nc = length(pitch),
+#'   formants = list(f1 = c(800, 1200), f2 = 2000, f3 = c(3500, 3200)),
+#'   lipRad = 0, temperature = .00001, plot = TRUE)
+#' formantSummary = t(data.frame(f1 = c(800, 1200), f2 = c(2000, 2000), f3 = c(3500, 3200)))
+#' pitch2 = soundgen:::lockToFormants(pitch = pitch, specEnv = specEnv,
+#'   rolloffMatrix = rolloffMatrix,
+#'   formantSummary = formantSummary,
+#'   lockProb = .5, minLength = 5, plot = TRUE)
+#' pitch3 = soundgen:::lockToFormants(pitch = pitch, specEnv = specEnv,
+#'   rolloffMatrix = rolloffMatrix,
+#'   formantSummary = formantSummary,
+#'   lockProb = list(time = c(0, .7, 1), value = c(0, 1, 0)),
+#'   minLength = 5, plot = TRUE)
+lockToFormants = function(pitch,
+                          specEnv,
+                          formantSummary,
+                          rolloffMatrix = NULL,
+                          lockProb = .1,
+                          minLength = 3,
+                          plot = FALSE) {
+  if (!is.null(rolloffMatrix)) {
+    nr = nrow(rolloffMatrix)
+  } else {
+    nr = 1
+    rolloffMatrix = matrix(1)
+  }
+  n = length(pitch)
+  if (is.list(lockProb)) {
+    lockProb = getSmoothContour(anchors = lockProb,
+                                len = n,
+                                valueFloor = permittedValues['formantLocking', 'low'],
+                                valueCeiling = permittedValues['formantLocking', 'high'])
+  }
+  freqs = as.numeric(rownames(specEnv)) * 1000
+  specEnv = interpolMatrix(
+    specEnv,
+    nr = nrow(specEnv),
+    nc = n,
+    interpol = 'approx'
+  )
+  # rownames(specEnv) = freqs
+  formants = interpolMatrix(as.matrix(formantSummary), nc = n)
+
+  # Calculate how much we can gain by locking a harmonic to a formant at each time point
+  d = data.frame(pitch = pitch)
+  d$pitch2 = d$pitch1 = d$pitch
+  d[, c('idx_max_gain', 'gain', 'nearest_formant')] = NA
+  for (i in 1:n){  # for each time point (normally glottal cycle in pitch_per_gc)
+    if (FALSE) {
+      plot(freqs, specEnv[, i], type = 'l')
+      abline(v = pitch[i], lty = 3)
+    }
+    # formants = as.data.frame(seewave::fpeaks(cbind(freqs, specEnv[, i]),
+    #                                          threshold = 1, plot = FALSE))
+
+    # for each harmonic in rolloffMatrix
+    temp = data.frame(harmonic = 1:nr, gain = NA, nearest_formant = NA)
+    for (h in 1:nr) {  # for each harmonic
+      nearest_formant = which.min(abs(formants[, i] - pitch[i] * h))
+      nearest_formant_amp = specEnv[which.min(abs(formants[nearest_formant, i] - freqs)), i]
+      idx_cur = which.min(abs(freqs - pitch[i] * h))
+      amp_gain = nearest_formant_amp / specEnv[idx_cur, i]
+      # amp_gain ~ how much E can be gained if locked to nearest formant
+      dist_to_nearest_formant = max(1, 12 * abs(log2(pitch[i] * h) - log2(formants[nearest_formant, i])))
+      # in semitones (anything closer than 1 semitone counts as 1 semitone to
+      # standardize penalization, otherwise we divide by very small numbers and
+      # thus inflate apparent gain)
+      temp$gain[h] = amp_gain / dist_to_nearest_formant * rolloffMatrix[h, i]
+      temp$nearest_formant[h] = formants[nearest_formant, i]
+    }
+    d$idx_max_gain[i] = which.max(temp$gain)
+    d$gain[i] = temp$gain[d$idx_max_gain[i]]
+    d$nearest_formant[i] = temp$nearest_formant[d$idx_max_gain[i]]
+    # the prob of locking to the nearest formant is some function of the gain in E
+    # penalized by dist_to_nearest_formant
+  }
+
+  # find the frames in which formant locking should be applied
+  lockThreshold = as.numeric(quantile(d$gain, probs = 1 - lockProb))  # can be a vector
+  lock_idx = which(d$gain > lockThreshold)
+  d$pitch1[lock_idx] = d$nearest_formant[lock_idx] / d$idx_max_gain[lock_idx]
+
+  # only preserve sufficiently long fragments of formant locking
+  d$pitch2 = d$pitch1
+  if (is.numeric(minLength) && minLength > 1) {
+    d$modif = (d$pitch != d$pitch1)
+    r = rle(d$modif)
+    r = data.frame(len = r[[1]], val = r[[2]], idx = 1)
+    if (r$len[1] < minLength) r$val[1] = FALSE
+    for (i in 2:nrow(r)) {
+      r$idx[i] = sum(r$len[1:(i - 1)]) + 1
+      if (r$len[i] < minLength) {
+        r$val[i] = FALSE
+      }
+    }
+    r1 = r[r$val == TRUE, , drop = FALSE]
+    d$modif1 = FALSE
+    #modif1 = soundgen:::clumper(modif, minLength = 5)
+    if (nrow(r1) >= 1) {
+      for (j in 1:nrow(r1)) {
+        d$modif1[r1$idx[j] : (r1$idx[j] + r1$len[j] - 1)] = TRUE
+      }
+    }
+    # + exclude rapid changes b/w the harmonic to which to lock? Tricky b/c we need to access different formant frequencies again, or find some intelligent way of removing pitch jumps
+
+    # un-modify short fragments, reverting to the original pitch
+    no_modif = which(d$modif1 == FALSE)
+    d$pitch2 = d$pitch1
+    d$pitch2[no_modif] = d$pitch[no_modif]
+  }
+
+
+  # # median smoothing to remove weird pitch jumps
+  # d$pitch2 = soundgen:::medianSmoother(data.frame(p = d$pitch1),
+  #                                             smoothing_ww = 30,
+  #                                             smoothingThres = 1)
+
+  if (plot) {
+    y_min = min(d[, c('pitch', 'pitch1', 'pitch2')]) / 1.1
+    y_max = max(d[, c('pitch', 'pitch1', 'pitch2')]) * 1.1
+    plot(d$pitch, type = 'l', lty = 2, col = 'red', ylim = c(y_min, y_max))
+    points(d$pitch1, type = 'l', lty =3, col = 'blue')
+    points(d$pitch2, type = 'l')
+    # for (f in 1:length(formants)) {
+    #   abline(h = formants[[f]]$freq, lty = 3)
+    # }
+  }
+
+  invisible(d$pitch2)
+}
