@@ -7,7 +7,7 @@
 #' \code{\link{spectrogram}}, take its logarithm (if \code{logSpec = TRUE}),
 #' center, perform a 2D Fourier transform (see also spec.fft() in the "spectral"
 #' package), take the upper half of the resulting symmetric matrix, and raise it
-#' to \code{power = 2}. The result is returned as \code{$original}. Roughness is
+#' to \code{power}. The result is returned as \code{$original}. Roughness is
 #' calculated as the proportion of energy / amplitude of the modulation spectrum
 #' within \code{roughRange} of temporal modulation frequencies. By default, the
 #' modulation matrix is then smoothed with Gaussian blur (see
@@ -41,6 +41,12 @@
 #'   FFT
 #' @param power raise modulation spectrum to this power (eg power = 2 for ^2, or
 #'   "power spectrum")
+#' @param returnComplex if TRUE, returns a complex modulation spectrum (without
+#'   normalization and warping)
+#' @param aggregComplex if TRUE, aggregates complex MS from multiple inputs,
+#'   otherwise returns the complex MS of the first input (recommended when
+#'   filtering and inverting the MS of a single sound, e.g. with
+#'   \code{\link{filterSoundByMS}})
 #' @param plot if TRUE, plots the modulation spectrum
 #' @param savePath if a valid path is specified, a plot is saved in this folder
 #'   (defaults to NA)
@@ -57,15 +63,17 @@
 #'   \code{\link[grDevices]{png}} if the plot is saved
 #' @param ... other graphical parameters passed on to \code{filled.contour.mod}
 #'   and \code{\link[graphics]{contour}} (see \code{\link{spectrogram}})
-#' @return Returns a list with three components:
+#' @return Returns a list with four components:
 #' \itemize{
 #' \item \code{$original} modulation spectrum prior to blurring and log-warping,
 #' but after squaring if \code{power = TRUE}, a matrix of nonnegative values.
-#' Rownames are temporal modulation frequencies (Hz), and colnames are
-#' spectral modulation frequencies (cycles/KHz).
+#' Rownames are spectral modulation frequencies (cycles/KHz), and colnames are
+#' temporal modulation frequencies (Hz).
 #' \item \code{$processed} modulation spectrum after blurring and log-warping
 #' \item \code{$roughness} proportion of energy / amplitude of the modulation
 #' spectrum within \code{roughRange} of temporal modulation frequencies, \%
+#' \item \code{$complex} untransformed complex modulation spectrum (returned
+#' only if returnComplex = TRUE)
 #' }
 #' @export
 #' @examples
@@ -90,8 +98,18 @@
 #'     pitch = runif(3, 400, 600))
 #' }
 #' # lapply(ss, playme)
-#' ms = modulationSpectrum(ss[[1]], samplingRate = 16000)  # the first sound
-#' ms = modulationSpectrum(ss, samplingRate = 16000)  # all 10 sounds
+#' ms1 = modulationSpectrum(ss[[1]], samplingRate = 16000)  # the first sound
+#' dim(ms1$original)
+#' ms2 = modulationSpectrum(ss, samplingRate = 16000)  # all 10 sounds
+#' dim(ms2$original)
+#'
+#' # Careful with complex MS of multiple inputs:
+#' ms3 = modulationSpectrum(ss, samplingRate = 16000,
+#'   returnComplex = TRUE, aggregComplex = FALSE)
+#' dim(ms3$complex)  # complex MS of the first input only
+#' ms4 = modulationSpectrum(ss, samplingRate = 16000,
+#'   returnComplex = TRUE, aggregComplex = TRUE)
+#' dim(ms4$complex)  # aggregated over inputs
 #'
 #' # As with spectrograms, there is a tradeoff in time-frequency resolution
 #' s = soundgen(pitch = 500, amFreq = 50, amDep = 100, samplingRate = 44100)
@@ -120,8 +138,6 @@
 #' ms = modulationSpectrum('~/Downloads/temp/', kernelSize = 11)
 #' # NB: longer files will be split into fragments <maxDur in length
 #'
-#' # "power = 2" returns squared modulation spectrum - note that this affects
-#' the roughness measure!
 #' # A sound with ~3 syllables per second and only downsweeps in F0 contour
 #' s = soundgen(nSyl = 8, sylLen = 200, pauseLen = 100, pitch = c(300, 200))
 #' # playme(s)
@@ -129,6 +145,9 @@
 #'   xlim = c(-25, 25), colorTheme = 'seewave', logWarp = NULL,
 #'   power = 2)
 #' # note the asymmetry b/c of downsweeps
+#'
+#' # "power = 2" returns squared modulation spectrum - note that this affects
+#' # the roughness measure!
 #' ms$roughness
 #' # compare:
 #' modulationSpectrum(s, samplingRate = 16000, maxDur = .5,
@@ -152,6 +171,11 @@
 #' # Log-transform the spectrogram prior to 2D FFT (affects roughness):
 #' ms = modulationSpectrum(soundgen(), samplingRate = 16000, logSpec = FALSE)
 #' ms = modulationSpectrum(soundgen(), samplingRate = 16000, logSpec = TRUE)
+#'
+#' # Complex modulation spectrum with phase preserved
+#' ms = modulationSpectrum(soundgen(), samplingRate = 16000,
+#'                         returnComplex = TRUE)
+#' image(t(log(abs(ms$complex))))
 #' }
 modulationSpectrum = function(
   x,
@@ -161,10 +185,12 @@ modulationSpectrum = function(
   windowLength = 25,
   step = NULL,
   overlap = 80,
-  wn = 'gaussian',
+  wn = 'hanning',
   zp = 0,
   power = 1,
   roughRange = c(30, 150),
+  returnComplex = FALSE,
+  aggregComplex = TRUE,
   plot = TRUE,
   savePath = NA,
   logWarp = 2,
@@ -224,7 +250,6 @@ modulationSpectrum = function(
   } else {
     stop('Input not recognized')
   }
-  if (is.null(step)) step = windowLength * (1 - overlap / 100)
 
   # load input
   duration = rep(NA, length(myInput))
@@ -249,6 +274,15 @@ modulationSpectrum = function(
     }
   }
 
+  # Re-set windowLength, step, and overlap so as to ensure that
+  # windowLength_points and step_points are not fractions
+  if (is.null(step)) step = windowLength * (1 - overlap / 100)
+  step_points = round(step / 1000 * samplingRate)
+  step = step_points[1] / samplingRate[1] * 1000
+  windowLength_points = round(windowLength / 1000 * samplingRate)
+  windowLength = windowLength_points[1] / samplingRate[1] * 1000
+  overlap = 100 * (1 - step_points[1] / windowLength_points[1])
+
   # split sounds that exceed maxDur
   toSplit = which(duration > maxDur)
   if (length(toSplit) > 0) {
@@ -262,6 +296,8 @@ modulationSpectrum = function(
         end = idx[j + 1]
         myInput[[length(myInput) + 1]] = myInput[[i]][start:end]
         samplingRate = c(samplingRate, samplingRate[i])
+        windowLength_points = c(windowLength_points, windowLength_points[length(windowLength_points)])
+        step_points = c(step_points, step_points[length(step_points)])
       }
       # the first fragment replaces the old long sound in myInput
       myInput[[i]] = myInput[[i]][1:idx[2]]
@@ -270,47 +306,77 @@ modulationSpectrum = function(
 
   # extract modulation spectrum per sound
   out = vector('list', length(myInput))
+  if (returnComplex) {
+    out_complex = out
+  } else {
+    out_aggreg_complex = NULL
+  }
   for (i in 1:length(myInput)) {
-    s1 = spectrogram(myInput[[i]],
-                     samplingRate = samplingRate[i],
-                     windowLength = windowLength,
-                     step = step,
-                     wn = wn,
-                     zp = zp,
-                     plot = FALSE,
-                     output = 'original',
-                     padWithSilence = FALSE)
-    # image(t(s1))
+    # s1 = spectrogram(myInput[[i]],
+    #                  samplingRate = samplingRate[i],
+    #                  windowLength = windowLength,
+    #                  step = step,
+    #                  wn = wn,
+    #                  zp = zp,
+    #                  plot = FALSE,
+    #                  output = 'original',
+    #                  padWithSilence = FALSE,
+    #                  normalize = TRUE)
+    # Calling stdft is ~80 times faster than going through spectrogram (!)
+    step_seq = seq(1, length(myInput[[i]]) + 1 - windowLength_points[i], step_points[i])
+    s1 = seewave::stdft(wave = as.matrix(myInput[[i]]),
+                        wn = wn,
+                        wl = windowLength_points[i],  # for multiple inputs, samplingRate, wl etc can vary
+                        f = samplingRate[i],
+                        zp = zp,
+                        step = step_seq,
+                        scale = TRUE,
+                        norm = FALSE,
+                        complex = FALSE)
+    # image(t(log(s1))); s1[1:3, 1:3]; dim(s1); range(s1)
     # log-transform amplitudes
     if (logSpec) {
-      s1 = log(s1)
+      positives = which(s1 > 0)
+      nonpositives = which(s1 <= 0)
+      s1[positives] = log(s1[positives])
+      if (length(positives) > 0 & length(nonpositives) > 0) {
+        s1[nonpositives] = min(s1[positives])
+      }
       s1 = s1 - min(s1) + 1e-16  # positive
+      # s1 = log(s1 + 1e-16)
+      # s1 = s1 - min(s1) + 1e-16  # positive
     }
-    # center - see spec.fft function in "spectral" package
-    s2 = s1 * (-1)^(row(s1) + col(s1))
     # 2D fft
-    s3 = abs(fft(s2, inverse = FALSE))
-    # image(t(s3))
-    # image(t(log(s3)))
-    s4 = s3[(nrow(s3) / 2 + 1) : nrow(s3), ]  # take only the upper half (always even)
+    ms_complex = specToMS(s1, windowLength = windowLength, step = step)
+    ms = abs(ms_complex)
+    # image(t(log(ms)))
+    symAxis = floor(nrow(ms) / 2) + 1
+    # ms[(symAxis - 2) : (symAxis + 2), 1:2]
+    ms_half = ms[symAxis:nrow(ms), ]  # take only the upper half (always even)
+
     # power
-    if (is.numeric(power)) s4 = s4 ^ power
+    if (is.numeric(power) && power != 1) ms_half = ms_half ^ power
+
     # normalize
-    s5 = s4 - min(s4)
-    s5 = s5 / max(s5)
-    # image(t(log(s5)))
-    out[[i]] = t(s5)
+    ms_half = ms_half - min(ms_half)
+    ms_half = ms_half / max(ms_half)
+    # image(t(log(ms_half)))
+    out[[i]] = ms_half
+
+    if (returnComplex) {
+      out_complex[[i]] = ms_complex
+    }
   }
 
   # average modulation spectra across all sounds
   max_rows = max(unlist(lapply(out, nrow)))
-  if (max_rows %% 2 == 0) max_rows = max_rows + 1  # make uneven to have a clear 0
   # normally same samplingRate, but in case not, upsample frequency resolution
   # typical ncol (depends on sound dur)
   typicalCols = round(median(unlist(lapply(out, ncol))))
   sr = max(samplingRate)  # again, in case not the same
   out1 = lapply(out, function(x) interpolMatrix(x, nr = max_rows, nc = typicalCols))
   out_aggreg = Reduce('+', out1) / length(myInput)
+  # image(t(log(out_aggreg)))
 
   # get time and frequency labels
   max_am = 1000 / step / 2
@@ -320,23 +386,38 @@ modulationSpectrum = function(
       'increase overlap / decrease step to improve temporal resolution,',
       'or else look for roughness in a lower range'))
   }
-  X = seq(-max_am, max_am, length.out = nrow(out_aggreg))  # time modulation
-  max_fm = ncol(out_aggreg) / (sr / 2 / 1000)
-  Y = seq(0, max_fm, length.out = ncol(out_aggreg))  # frequency modulation
-  rownames(out_aggreg) = X
-  colnames(out_aggreg) = Y
+  X = seq(-max_am, max_am, length.out = ncol(out_aggreg))  # time modulation
+  max_fm = nrow(out_aggreg) / (sr / 2 / 1000)
+  Y = seq(max_fm / nrow(out_aggreg), max_fm, length.out = nrow(out_aggreg))  # frequency modulation
+  rownames(out_aggreg) = Y
+  colnames(out_aggreg) = X
+
+  # prepare a separate summary of the complex ms
+  if (returnComplex) {
+    if (aggregComplex) {
+      out1_complex = lapply(out_complex, function(x) {
+        interpolMatrix(x, nr = max_rows * 2, nc = typicalCols)
+      })
+      out_aggreg_complex = Reduce('+', out1_complex) / length(myInput)
+      colnames(out_aggreg_complex) = X
+      rownames(out_aggreg_complex) = c(-rev(Y), Y)
+      # image(t(log(abs(out_aggreg_complex))))
+    } else {
+      out_aggreg_complex = out_complex[[1]]
+    }
+  }
 
   # extract a measure of roughness
   roughness = getRough(out_aggreg, roughRange)
 
-  # log-transform the axes (or, actually, warp the matrix itself)
+  # log-transform the axes (or, actually, warps the matrix itself)
   if (is.numeric(logWarp)) {
-    zero_row = ceiling(max_rows / 2)
-    m_left = logMatrix(out_aggreg[zero_row:1, ], base = logWarp)  # NB: flip the left half!
-    m_right = logMatrix(out_aggreg[zero_row : max_rows, ], base = logWarp)
-    out_transf = rbind(m_left[nrow(m_left):1, ], m_right[2:nrow(m_right), ])
-    X1 = as.numeric(rownames(out_transf))  # warped by logMatrix
-    Y1 = as.numeric(colnames(out_transf))  # warped by logMatrix
+    zero_col = ceiling(ncol(out_aggreg) / 2)
+    m_left = logMatrix(out_aggreg[, zero_col:1], base = logWarp)  # NB: flip the left half!
+    m_right = logMatrix(out_aggreg[, zero_col:ncol(out_aggreg)], base = logWarp)
+    out_transf = cbind(m_left[, ncol(m_left):1], m_right[, 2:ncol(m_right)])
+    X1 = as.numeric(colnames(out_transf))  # warped by logMatrix
+    Y1 = as.numeric(rownames(out_transf))  # warped by logMatrix
   } else {
     out_transf = out_aggreg
   }
@@ -363,7 +444,7 @@ modulationSpectrum = function(
     color.palette = switchColorTheme(colorTheme)
     if (is.numeric(logWarp)) {
       filled.contour.mod(
-        x = X, y = Y, z = out_transf,
+        x = X, y = Y, z = t(out_transf),
         levels = seq(0, 1, length = 30),
         color.palette = color.palette,
         xlab = xlab, ylab = ylab,
@@ -419,7 +500,7 @@ modulationSpectrum = function(
       #      labels = yl)
     } else {
       filled.contour.mod(
-        x = X, y = Y, z = out_transf,
+        x = X, y = Y, z = t(out_transf),
         levels = seq(0, 1, length = 30),
         color.palette = color.palette,
         xlab = xlab, ylab = ylab,
@@ -431,7 +512,7 @@ modulationSpectrum = function(
     # qntls = quantile(out_aggreg, probs = quantiles)  # could try HDI instead
     qntls = pDistr(as.numeric(out_transf), quantiles = quantiles)
     par(new = TRUE)
-    contour(x = X, y = Y, z = out_transf,
+    contour(x = X, y = Y, z = t(out_transf),
             levels = qntls, labels = quantiles * 100,
             xaxs = 'i', yaxs = 'i',
             axes = FALSE, frame.plot = FALSE,
@@ -442,6 +523,7 @@ modulationSpectrum = function(
 
   invisible(list('original' = out_aggreg,
                  'processed' = out_transf,
+                 'complex' = out_aggreg_complex,
                  'roughness' = roughness))
 }
 
@@ -476,7 +558,7 @@ modulationSpectrumFolder = function(
   windowLength = 25,
   step = NULL,
   overlap = 80,
-  wn = 'gaussian',
+  wn = 'hamming',
   zp = 0,
   power = 1,
   roughRange = c(30, 150),
@@ -544,22 +626,21 @@ modulationSpectrumFolder = function(
 #'
 #' Helper function for calculating roughness - the proportion of energy /
 #' amplitude in the roughness range
-#' @param m numeric matrix of non-negative values with rownames giving temporal
+#' @param m numeric matrix of non-negative values with colnames giving temporal
 #'   modulation frequency
 #' @param roughRange range of temporal modulation frequencies corresponding to
 #'   roughness
 #' @return Returns roughness in percent.
 #' @keywords internal
 #' @examples
-#' m = matrix(rnorm(100, 10, 1), nrow = 10)
-#' rownames(m) = seq(-10, 10, length.out = nrow(m))
-#' soundgen:::getRough(m, roughRange = c(6, 8))
+#' m = modulationSpectrum(soundgen(), samplingRate = 16000)$original
+#' soundgen:::getRough(m, roughRange = c(30, Inf))
 getRough = function(m, roughRange) {
-  rn = abs(as.numeric(rownames(m)))
-  rough_rows = which(rn > roughRange[1] &
-                       rn < roughRange[2])
-  if (length(rough_rows) > 0) {
-    roughness = sum(m[rough_rows, ]) / sum(m) * 100
+  colNames = abs(as.numeric(colnames(m)))
+  rough_cols = which(colNames > roughRange[1] &
+                       colNames < roughRange[2])
+  if (length(rough_cols) > 0) {
+    roughness = sum(m[, rough_cols]) / sum(m) * 100
   } else {
     roughness = 0
   }
